@@ -29,7 +29,6 @@ sub new {
     my $error;
     if ($version eq 'snmpv3') {
         ($self->{session}, $error) = Net::SNMP->session(
-            -timeout      => 15,
             -retries      => 0,
             -version      => $version,
             -hostname     => $params{hostname},
@@ -41,7 +40,6 @@ sub new {
         );
     } else { # snmpv2c && snmpv1 #
         ($self->{session}, $error) = Net::SNMP->session(
-            -timeout   => 1,
             -retries   => 0,
             -version   => $version,
             -hostname  => $params{hostname},
@@ -51,6 +49,8 @@ sub new {
 
     die $error unless $self->{session};
 
+    $self->{session}->timeout($params{timeout}) if $params{timeout};
+
     bless $self, $class;
 
     return $self;
@@ -59,13 +59,21 @@ sub new {
 sub switch_community {
     my ($self, $suffix) = @_;
 
-    ($self->{session}, undef) = Net::SNMP->session(
-            -timeout   => 1,
+    my $version_id = $self->{session}->version();
+    my $version =
+        $version_id == 0 ? 'snmpv1'  :
+        $version_id == 1 ? 'snmpv2c' :
+        $version_id == 2 ? 'snmpv3'  :
+                             undef   ;
+    my $error;
+    ($self->{session}, $error) = Net::SNMP->session(
+            -timeout   => $self->{session}->timeout(),
             -retries   => 0,
-            -version   => $self->{session}->version(),
+            -version   => $version,
             -hostname  => $self->{session}->hostname(),
             -community => $self->{community} . $suffix
     );
+    die $error unless $self->{session};
 }
 
 sub get {
@@ -151,6 +159,10 @@ Can be one of:
 
 =back
 
+=item timeout
+
+The transport layer timeout
+
 =item hostname (mandatory)
 
 =item community
