@@ -70,6 +70,7 @@ sub run {
     my $options     = $self->{options};
     my $pid         = $options->{PARAM}->[0]->{PID};
     my $max_threads = $options->{PARAM}->[0]->{THREADS_QUERY};
+    my $timeout     = $options->{PARAM}->[0]->{TIMEOUT};
 
     # SNMP models
     my $models = _getIndexedModels($options->{MODEL});
@@ -102,6 +103,7 @@ sub run {
             \@results,
             $models,
             $credentials,
+            $timeout,
         )->detach();
     }
 
@@ -160,7 +162,7 @@ sub _sendMessage {
 }
 
 sub _queryDevices {
-    my ($self, $state, $devices, $results, $models, $credentials) = @_;
+    my ($self, $state, $devices, $results, $models, $credentials, $timeout) = @_;
 
     my $logger = $self->{logger};
     my $id     = threads->tid();
@@ -179,6 +181,7 @@ sub _queryDevices {
 
         my $result = $self->_queryDevice(
             device      => $device,
+            timeout     => $timeout,
             model       => $models->{$device->{MODELSNMP_ID}},
             credentials => $credentials->{$device->{AUTHSNMP_ID}}
         );
@@ -230,6 +233,7 @@ sub _queryDevice {
             $snmp = FusionInventory::Agent::SNMP::Live->new(
                 version      => $credentials->{VERSION},
                 hostname     => $device->{IP},
+                timeout      => $params{timeout} || 15,
                 community    => $credentials->{COMMUNITY},
                 username     => $credentials->{USERNAME},
                 authpassword => $credentials->{AUTHPASSWORD},
@@ -245,11 +249,12 @@ sub _queryDevice {
     }
 
     my $result = getDeviceFullInfo(
-         id     => $device->{ID},
-         type   => $device->{TYPE},
-         snmp   => $snmp,
-         model  => $params{model},
-         logger => $self->{logger}
+         id      => $device->{ID},
+         type    => $device->{TYPE},
+         snmp    => $snmp,
+         model   => $params{model},
+         logger  => $self->{logger},
+         datadir => $self->{datadir}
     );
 
     return $result;
