@@ -73,10 +73,12 @@ sub getDmidecodeInfos {
         next unless $line =~ /^\s+ ([^:]+) : \s (.*\S)/x;
 
         next if
-            $2 eq 'N/A'           ||
-            $2 eq 'Not Specified' ||
-            $2 eq '<BAD INDEX>'   ||
-            $2 eq 'Not Present'   ;
+            $2 eq 'N/A'                        ||
+            $2 eq 'Not Specified'              ||
+            $2 eq 'Not Present'                ||
+            $2 eq '<BAD INDEX>'                ||
+            $2 eq '<OUT OF SPEC>'              ||
+            $2 eq '<OUT OF SPEC><OUT OF SPEC>' ;
 
         $block->{$1} = $2;
     }
