@@ -37,7 +37,7 @@ my %sysobjectid_vendors = (
     36    => { vendor => 'DEC',             type => 'COMPUTER'   },
     42    => { vendor => 'Sun',             type => 'COMPUTER'   },
     43    => { vendor => '3Com',            type => 'NETWORKING' },
-    45    => { vendor => 'SynOptics',       type => 'NETWORKING' },
+    45    => { vendor => 'Nortel',          type => 'NETWORKING' },
     63    => { vendor => 'Apple',                                },
     171   => { vendor => 'D-Link',          type => 'NETWORKING' },
     186   => { vendor => 'Toshiba',         type => 'PRINTER'    },
@@ -57,11 +57,13 @@ my %sysobjectid_vendors = (
     1602  => { vendor => 'Canon',           type => 'PRINTER'    },
     1805  => { vendor => 'Sagem',          type => 'NETWORKING' },
     1872  => { vendor => 'Alteon',          type => 'NETWORKING' },
-    1916  => { vendor => 'Extrem Networks', type => 'NETWORKING' },
+    1916  => { vendor => 'Extreme',         type => 'NETWORKING' },
+    1981  => { vendor => 'EMC'                                   },
     1991  => { vendor => 'Foundry',         type => 'NETWORKING' },
     2385  => { vendor => 'Sharp',           type => 'PRINTER'    },
     2435  => { vendor => 'Brother',         type => 'PRINTER'    },
     2636  => { vendor => 'Juniper',         type => 'NETWORKING' },
+    3224  => { vendor => 'NetScreen',       type => 'NETWORKING' },
     3977  => { vendor => 'Broadband',       type => 'NETWORKING' },
     5596  => { vendor => 'Tandberg'                              },
     6486  => { vendor => 'Alcatel',         type => 'NETWORKING' },
@@ -71,6 +73,8 @@ my %sysobjectid_vendors = (
     18334 => { vendor => 'Konica',          type => 'PRINTER'    },
 );
 
+my %sysobjectid_models;
+
 my %sysdescr_first_word = (
     '3com'           => { vendor => '3Com',            type => 'NETWORKING' },
     'alcatel-lucent' => { vendor => 'Alcatel-Lucent',  type => 'NETWORKING' },
@@ -112,7 +116,7 @@ my %sysdescr_first_word = (
     'officejet'      => { vendor => 'Hewlett-Packard', type => 'PRINTER'    },
     'oki'            => { vendor => 'OKI',             type => 'PRINTER'    },
     'powerconnect'   => { vendor => 'PowerConnect',    type => 'NETWORKING' },
-    'procurve'       => { vendor => 'Hewlett Packard', type => 'NETWORKING' },
+    'procurve'       => { vendor => 'Hewlett-Packard', type => 'NETWORKING' },
     'ricoh'          => { vendor => 'Ricoh',           type => 'PRINTER'    },
     'sagem'          => { vendor => 'Sagem',           type => 'NETWORKING' },
     'samsung'        => { vendor => 'Samsung',         type => 'PRINTER'    },
@@ -247,14 +251,6 @@ my @connected_devices_rules = (
     },
 );
 
-my @specific_cleanup_rules = (
-    {
-        match    => qr/3Com IntelliJack/,
-        module   => 'FusionInventory::Agent::Tools::Hardware::3Com',
-        function => 'RewritePortOf225'
-    },
-);
-
 # common base variables
 my %base_variables = (
     MAC          => 'macaddr',
@@ -353,7 +349,7 @@ my %printer_pagecounters_variables = (
 );
 
 sub getDeviceBaseInfo {
-    my ($snmp) = @_;
+    my ($snmp, $datadir) = @_;
 
     # retrieve sysdescr value, as it is our primary identification key
     my $sysdescr = $snmp->get('.1.3.6.1.2.1.1.1.0'); # SNMPv2-MIB::sysDescr.0
@@ -366,16 +362,26 @@ sub getDeviceBaseInfo {
     # first heuristic:
     # compute manufacturer and type from sysobjectid (SNMPv2-MIB::sysObjectID.0)
     my $sysobjectid = $snmp->get('.1.3.6.1.2.1.1.2.0');
-    my $vendor_id =
-        $sysobjectid =~ /^SNMPv2-SMI::enterprises\.(\d+)/ ? $1 :
-        $sysobjectid =~ /^iso\.3\.6\.1\.4\.1\.(\d+)/      ? $1 :
-        $sysobjectid =~ /^\.1\.3\.6\.1\.4\.1\.(\d+)/      ? $1 :
-                                                            undef;
-    if ($vendor_id) {
-        my $result = $sysobjectid_vendors{$vendor_id};
-        if ($result) {
-            $device{MANUFACTURER} = $result->{vendor};
-            $device{TYPE}         = $result->{type} if $result->{type};
+    if ($sysobjectid) {
+        my $prefix = qr/(?:
+            SNMPv2-SMI::enterprises |
+            iso\.3\.6\.1\.4\.1      |
+            \.1\.3\.6\.1\.4\.1
+        )/x;
+        my ($vendor_id, $model_id) =
+            $sysobjectid =~ /^ $prefix \. (\d+) (?: \. (.+) )? $/x;
+        if ($vendor_id) {
+            my $result = $sysobjectid_vendors{$vendor_id};
+            if ($result) {
+                $result->{model} = _getDeviceModel(
+                    vendor  => $result->{vendor},
+                    id      => $model_id,
+                    datadir => $datadir,
+                );
+                $device{MANUFACTURER} = $result->{vendor};
+                $device{TYPE}         = $result->{type}  if $result->{type};
+                $device{MODEL}        = $result->{model} if $result->{model};
+            }
         }
     }
 
@@ -416,6 +422,41 @@ sub getDeviceBaseInfo {
     return %device;
 }
 
+sub _getDeviceModel {
+    my (%params) = @_;
+
+    return unless $params{id};
+
+    # load vendor-specific database if not already done
+    my $vendor = lc($params{vendor});
+    $vendor =~ s/ /_/g;
+    $sysobjectid_models{$vendor} = _loadDeviceModels(
+        file => "$params{datadir}/sysobjectid.$vendor.ids"
+    ) if !exists $sysobjectid_models{$vendor};
+
+    return $sysobjectid_models{$vendor}->{$params{id}};
+}
+
+sub _loadDeviceModels {
+    my (%params) = @_;
+
+    my $handle = getFileHandle(%params);
+    return unless $handle;
+
+    my $models;
+
+    while (my $line = <$handle>) {
+        chomp $line;
+        my ($id, $name) = split(/\t/, $line);
+        next unless $id;
+        $models->{$id} = $name;
+    }
+
+    close $handle;
+
+    return $models;
+}
+
 sub _getSerial {
     my ($snmp, $model) = @_;
 
@@ -449,11 +490,14 @@ sub _getMacAddress {
 }
 
 sub getDeviceInfo {
-     my ($snmp, $dictionary) = @_;
+    my (%params) = @_;
+
+    my $snmp       = $params{snmp};
+    my $dictionary = $params{dictionary};
 
     # the device is initialized with basic information
     # deduced from its sysdescr
-    my %device = getDeviceBaseInfo($snmp);
+    my %device = getDeviceBaseInfo($snmp, $params{datadir});
     return unless %device;
 
     # then, we try to get a matching model from the dictionary,
@@ -476,7 +520,6 @@ sub getDeviceInfo {
         $device{SERIAL}    = _getSerial($snmp, $model);
         $device{MODELSNMP} = $model->{MODELSNMP};
         $device{FIRMWARE}  = $model->{FIRMWARE};
-        $device{MODEL}     = $model->{MODEL};
     } else {
         # otherwise, we fallback on default mappings
         $device{MAC} = _getMacAddress($snmp);
@@ -531,7 +574,7 @@ sub _setTrunkPorts {
     }
 
 }
-sub _setConnectedDevices {
+sub _setConnectedDevicesInfo {
     my ($description, $snmp, $model, $ports, $logger) = @_;
 
     foreach my $rule (@connected_devices_rules) {
@@ -539,7 +582,7 @@ sub _setConnectedDevices {
 
         runFunction(
             module   => $rule->{module},
-            function => 'setConnectedDevices',
+            function => 'setConnectedDevicesInfo',
             params   => {
                 snmp   => $snmp,
                 model  => $model,
@@ -553,27 +596,6 @@ sub _setConnectedDevices {
     }
 }
 
-sub _performSpecificCleanup {
-    my ($description, $snmp, $model, $ports) = @_;
-
-    foreach my $rule (@specific_cleanup_rules) {
-        next unless $description =~ $rule->{match};
-
-        runFunction(
-            module   => $rule->{module},
-            function => $rule->{function},
-            params   => {
-                snmp    => $snmp,
-                model   => $model,
-                ports   => $ports
-            },
-            load     => 1
-        );
-
-        last;
-    }
-}
-
 sub getDeviceFullInfo {
     my (%params) = @_;
 
@@ -582,7 +604,7 @@ sub getDeviceFullInfo {
     my $logger = $params{logger};
 
     # first, let's retrieve basic device informations
-    my %info = getDeviceBaseInfo($snmp);
+    my %info = getDeviceBaseInfo($snmp, $params{datadir});
     return unless %info;
 
     # unfortunatly, some elements differs between discovery
@@ -750,7 +772,8 @@ sub _setPrinterProperties {
     my $snmp   = $params{snmp};
     my $model  = $params{model};
 
-    $device->{INFO}->{MODEL} = $snmp->get($model->{oids}->{model});
+    $device->{INFO}->{MODEL} = $snmp->get($model->{oids}->{model})
+        if !$device->{INFO}->{MODEL};
 
     # consumable levels
     foreach my $key (keys %printer_cartridges_simple_variables) {
@@ -803,7 +826,8 @@ sub _setNetworkingProperties {
     my $model  = $params{model};
     my $logger = $params{logger};
 
-    $device->{INFO}->{MODEL} = $snmp->get($model->{oids}->{entPhysicalModelName});
+    $device->{INFO}->{MODEL} = $snmp->get($model->{oids}->{entPhysicalModelName})
+        if !$device->{INFO}->{MODEL};
 
     my $comments = $device->{INFO}->{DESCRIPTION} || $device->{INFO}->{COMMENTS};
     my $ports    = $device->{PORTS}->{PORT};
@@ -840,7 +864,7 @@ sub _setNetworkingProperties {
 
     _setTrunkPorts($comments, $snmp, $model, $ports, $logger);
 
-    _setConnectedDevices($comments, $snmp, $model, $ports, $logger);
+    _setConnectedDevicesInfo($comments, $snmp, $model, $ports, $logger);
 
     # check if vlan-specific queries are needed
     my $vlan_query =
@@ -869,9 +893,6 @@ sub _setNetworkingProperties {
             logger => $logger
         );
     }
-
-    # hardware-specific hacks
-    _performSpecificCleanup($comments, $snmp, $model, $ports);
 }
 
 sub _getPercentValue {
@@ -939,19 +960,23 @@ sub getCanonicalMacAddress {
 
     return unless $value;
 
+    my $r;
     if ($value =~ /$mac_address_pattern/) {
         # this was stored as a string, it just has to be normalized
-        return join(':', map { sprintf "%02X", hex($_) } split(':', $value));
+        $r = join(':', map { sprintf "%02X", hex($_) } split(':', $value));
     } else {
         # this was stored as an hex-string
-        if ($value =~ /^0x/) {
+        # 0xD205A86C26D5 or 0x6001D205A86C26D5
+        if ($value =~ /^0x[0-9A-F]{0,4}([0-9A-F]{12})$/i) {
             # value translated by Net::SNMP
-            return alt2canonical($value);
+            $r = alt2canonical('0x'.$1);
         } else {
             # packed value, onvert from binary to hexadecimal
-            return unpack 'H*', $value;
+            $r = getCanonicalMacAddress("0x".unpack 'H*', $value);
         }
     }
+
+    return $r;
 }
 
 sub getCanonicalSerialNumber {
@@ -1010,7 +1035,7 @@ This module provides some hardware-related functions.
 return a minimal set of information for a device through SNMP, according to a
 set of rules hardcoded in the agent.
 
-=head2 getDeviceInfo($snmp, $dictionary)
+=head2 getDeviceInfo(%params)
 
 return a limited set of information for a device through SNMP, according to a
 set of rules hardcoded in the agent and the usage of generic knowledge base,
@@ -1021,70 +1046,6 @@ the dictionary.
 return a full set of information for a device through SNMP, according to a
 set of rules hardcoded in the agent and the usage of a device-specific set of mappings, the model.
 
-=head2 setConnectedDevicesMacAddresses($description, $snmp, $model, $ports)
-
-set mac addresses of connected devices.
-
-=over
-
-=item * description: device identification key
-
-=item * snmp: FusionInventory::Agent::SNMP object
-
-=item * model: SNMP model
-
-=item * ports: device ports list
-
-=back
-
-=head2 setConnectedDevices($description, $snmp, $model, $ports)
-
-Set connected devices using CDP if available, LLDP otherwise.
-
-=over
-
-=item * description: device identification key
-
-=item * snmp: FusionInventory::Agent::SNMP object
-
-=item * model: SNMP model
-
-=item * ports: device ports list
-
-=back
-
-=head2 setTrunkPorts($description, $snmp, $model, $ports)
-
-Set trunk flag on ports needing it.
-
-=over
-
-=item * description: device identification key
-
-=item * snmp: FusionInventory::Agent::SNMP object
-
-=item * model: SNMP model
-
-=item * ports: device ports list
-
-=back
-
-=head2 performSpecificCleanup($description, $snmp, $model, $ports)
-
-Perform device-specific miscaelanous cleanups
-
-=over
-
-=item * description: device identification key
-
-=item * snmp: FusionInventory::Agent::SNMP object
-
-=item * model: SNMP model
-
-=item * ports: device ports list
-
-=back
-
 =head2 getCanonicalSerialNumber($value)
 
 Return a canonical value for a serial number.
